from Products.CMFCore.utils import getToolByName

transform = 'avi_to_flv'

def installTransform(portal):
    """
    Install avi2flv transform.
    """
    transforms = getToolByName(portal, 'portal_transforms')
    if transform not in transforms.objectIds():
        transforms.manage_addTransform(
            transform,
            'stxnext.transform.avi2flv.%s' % transform,
        )

def uninstallTransform(portal):
    """
    Uninstall avi2flv transform.
    """
    transforms = getToolByName(portal, 'portal_transforms')
    if transform in transforms.objectIds():
        transforms.manage_delObjects([transform])

def importVarious(context):
    """
    Various import step code.
    """
    marker_file = 'stxnext.transform.avi2flv.txt'
    if context.readDataFile(marker_file) is None:
        ## don't install if called as dependency
        return
    portal = context.getSite()
    installTransform(portal)

def importVariousUninstall(context):
    """
    Various uninstall step code.
    """
    marker_file = 'stxnext.transform.avi2flv-uninstall.txt'
    if context.readDataFile(marker_file) is None:
        ## don't uninstall if called as dependency
        return
    portal = context.getSite()
    uninstallTransform(portal)



from Products.CMFCore.utils import getToolByName

def install(portal):
    setup_tool = getToolByName(portal, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile('profile-stxnext.transform.avi2flv:default')
    return "Ran all import steps."

def uninstall(portal):
    setup_tool = getToolByName(portal, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile('profile-stxnext.transform.avi2flv:uninstall')
    return "Ran all uninstall steps."



import os, subprocess, signal, tempfile

FFMPEG_OPTS = '-y -b 1024k -r 25 -acodec libmp3lame -ar 44100'

def converter(avi_data, ffmpeg_opts=None):
    """
    Execute ffmpeg and return FLV file data.
    
    Function operates on temporary files instead of pipes,
    pipes are constantly raising "broken pipe" exceptions
    because of weird python implementation.
    """
    if not ffmpeg_opts:
        ffmpeg_opts = FFMPEG_OPTS
    ffmpeg_cmd = 'ffmpeg -i "%%s" %s "%%s"' % ffmpeg_opts

    in_name = tempfile.mkstemp(suffix='.avi')[1]
    in_file = open(in_name, 'wb')
    in_file.write(avi_data)
    in_file.close()
    out_name = tempfile.mkstemp(suffix='.flv')[1]

    ps = subprocess.Popen(ffmpeg_cmd % (in_name , out_name),
                          bufsize= -1,
                          stderr=open('/dev/null', 'w'),
                          preexec_fn=lambda: signal.signal(signal.SIGPIPE, signal.SIG_DFL),
                          shell=True)
    ps.communicate()

    out_file = open(out_name)
    flv_data = out_file.read()
    out_file.close()

    os.remove(in_name)
    os.remove(out_name)

    return flv_data




from Products.CMFCore.utils import getToolByName

def convert_fileupload(value, context):
    if value:
        portal_transforms = getToolByName(context, 'portal_transforms')
        if 'avi_to_flv' in portal_transforms and value.headers.get('content-type') in portal_transforms.avi_to_flv.inputs:
            flv_data = portal_transforms.convert('avi_to_flv', value.read()).getData()

            ## replace AVI content with new FLV
            value.seek(0)
            value.truncate(0)
            value.write(flv_data)
            value.seek(0)

            ## update content type and filename
            value.headers['content-type'] = 'video/x-flv'
            if value.filename.endswith('AVI') or value.filename.endswith('avi'):
                value.filename = value.filename[:-3] + 'flv'

    return value

def patched_setFile(self, value, **kwargs):
    """
    Convert AVI fo FLV if uploaded file has recognized content type.
    """
    value = convert_fileupload(value, context=self)
    self._setATCTFileContent(value, **kwargs)



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

version = open('src/stxnext/transform/avi2flv/version.txt').read()

setup (
    name='stxnext.transform.avi2flv',
    version=version,
    author='STX Next Sp. z o.o, Wojciech Lichota',
    author_email='info@stxnext.pl',
    description='Converts clips from AVI format to FLV during upload to Plone.',
    long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
    keywords='plone avi flv movie clip flash video',
    platforms=['any'],
    url='http://www.stxnext.pl/open-source',
    license='Zope Public License, Version 2.1 (ZPL)',
    packages=find_packages('src'),
    include_package_data=True,
    package_dir={'':'src'},
    namespace_packages=['stxnext'],
    zip_safe=False,

    install_requires=[
        'setuptools',
        'collective.monkeypatcher',
       ],

    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Zope2',
        'Framework :: Plone',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Natural Language :: English',
        'Programming Language :: Python',
        ]
    )



from Products.PortalTransforms.interfaces import itransform

from avi2flv import converter, FFMPEG_OPTS


class Avi2Flv(object):
    """
    Transform which converts from AVI fo FLV.
    """
    __implements__ = itransform

    __name__ = 'avi_to_flv'
    output = 'video/x-flv'

    def __init__(self, name=None, inputs=('video/x-avi', 'video/x-msvideo'), ffmpeg_opts=FFMPEG_OPTS):
        self.config = {
            'inputs' : inputs,
            'ffmpeg_opts': ffmpeg_opts,
        }
        self.config_metadata = {
            'inputs': ('list',
                       'Inputs',
                       'Input(s) MIME type. Change with care.'),
            'ffmpeg_opts': ('string',
                            'FFmpeg options',
                            'FFmpeg convert options'),
            }
        if name:
            self.__name__ = name

    def name(self):
        return self.__name__

    def __getattr__(self, attr):
        if attr in self.config:
            return self.config[attr]
        raise AttributeError(attr)

    def convert(self, avi_data, data, **kwargs):
        flv_data = converter(avi_data, self.config.get('ffmpeg_opts', FFMPEG_OPTS))
        data.setData(flv_data)
        return data


def register():
    return Avi2Flv()
